@@ -1,26 +1,16 @@
+// SPDX-License-Identifier: MIT
 /*
-    QmVk - simple Vulkan library created for QMPlay2
-    Copyright (C) 2020-2021  Błażej Szczygieł
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published
-    by the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   QmVk - simple Vulkan library created for QMPlay2
+   Copyright (C) 2020-2022 Błażej Szczygieł
 */
 
 #include "MemoryObjectDescr.hpp"
 #include "Buffer.hpp"
-#include "Image.hpp"
 #include "BufferView.hpp"
-#include "Sampler.hpp"
+#ifndef QMVK_NO_GRAPHICS
+#   include "Image.hpp"
+#   include "Sampler.hpp"
+#endif
 
 namespace QmVk {
 
@@ -36,17 +26,20 @@ static vector<shared_ptr<MemoryObjectBase>> toMemoryObjectBaseVector(const vecto
 
 MemoryObjectDescr::MemoryObjectDescr(
     const vector<shared_ptr<Buffer>> &buffers,
-    Access access)
+    Access access,
+    const vector<BufferRange> &ranges)
     : m_type(Type::Buffer)
     , m_access(access)
     , m_objects(toMemoryObjectBaseVector(buffers))
-    , m_descriptorTypeInfos(getBufferDescriptorTypeInfos())
+    , m_descriptorTypeInfos(getBufferDescriptorTypeInfos(ranges))
 {}
+#ifndef QMVK_NO_GRAPHICS
 MemoryObjectDescr::MemoryObjectDescr(
     const vector<shared_ptr<Image>> &images,
     const shared_ptr<Sampler> &sampler,
     uint32_t plane)
     : m_type(Type::Image)
+    , m_access(Access::Read)
     , m_objects(toMemoryObjectBaseVector(images))
     , m_sampler(sampler)
     , m_plane(plane)
@@ -62,6 +55,7 @@ MemoryObjectDescr::MemoryObjectDescr(
     , m_plane(plane)
     , m_descriptorTypeInfos(getImageDescriptorTypeInfos())
 {}
+#endif
 MemoryObjectDescr::MemoryObjectDescr(
     const vector<shared_ptr<BufferView>> &bufferViews,
     Access access)
@@ -73,17 +67,20 @@ MemoryObjectDescr::MemoryObjectDescr(
 
 MemoryObjectDescr::MemoryObjectDescr(
     const shared_ptr<Buffer> &buffer,
-    Access access)
+    Access access,
+    const BufferRange &range)
     : m_type(Type::Buffer)
     , m_access(access)
     , m_objects({buffer})
-    , m_descriptorTypeInfos(getBufferDescriptorTypeInfos())
+    , m_descriptorTypeInfos(getBufferDescriptorTypeInfos({range}))
 {}
+#ifndef QMVK_NO_GRAPHICS
 MemoryObjectDescr::MemoryObjectDescr(
     const shared_ptr<Image> &image,
     const shared_ptr<Sampler> &sampler,
     uint32_t plane)
     : m_type(Type::Image)
+    , m_access(Access::Read)
     , m_objects({image})
     , m_sampler(sampler)
     , m_plane(plane)
@@ -99,6 +96,7 @@ MemoryObjectDescr::MemoryObjectDescr(
     , m_plane(plane)
     , m_descriptorTypeInfos(getImageDescriptorTypeInfos())
 {}
+#endif
 MemoryObjectDescr::MemoryObjectDescr(
     const shared_ptr<BufferView> &bufferView,
     Access access)
@@ -108,46 +106,87 @@ MemoryObjectDescr::MemoryObjectDescr(
     , m_descriptorTypeInfos(getBufferViewDescriptorTypeInfos())
 {}
 
-void MemoryObjectDescr::prepareImage(
+void MemoryObjectDescr::prepareObject(
     vk::CommandBuffer commandBuffer,
     vk::PipelineStageFlags pipelineStageFlags) const
 {
-    if (m_type != Type::Image)
-        return;
+    vk::AccessFlags accessFlag = {};
+    switch (m_access)
+    {
+        case Access::Read:
+        case Access::StorageRead:
+            accessFlag = vk::AccessFlagBits::eShaderRead;
+            break;
+        case Access::Write:
+        case Access::StorageWrite:
+            accessFlag = vk::AccessFlagBits::eShaderWrite;
+            break;
+        case Access::Storage:
+            accessFlag = vk::AccessFlagBits::eShaderRead | vk::AccessFlagBits::eShaderWrite;
+            break;
+    }
 
+#ifndef QMVK_NO_GRAPHICS
     size_t descriptorInfosIdx = 0;
+#endif
     for (auto &&object : m_objects)
     {
-        auto image = static_pointer_cast<Image>(object);
-        image->pipelineBarrier(
-            commandBuffer,
-            descriptorInfos()[descriptorInfosIdx].descrImgInfo.imageLayout,
-            pipelineStageFlags,
-            (m_access == Access::Write)
-                ? vk::AccessFlagBits::eShaderWrite
-                : vk::AccessFlagBits::eShaderRead
-        );
-        descriptorInfosIdx += (m_plane == ~0u)
-            ? image->numPlanes()
-            : 1
-        ;
+        switch (m_type)
+        {
+            case Type::Buffer:
+            case Type::BufferView:
+            {
+                auto buffer = (m_type == Type::BufferView)
+                    ? static_pointer_cast<BufferView>(object)->buffer()
+                    : static_pointer_cast<Buffer>(object)
+                ;
+                buffer->pipelineBarrier(
+                    commandBuffer,
+                    pipelineStageFlags,
+                    accessFlag
+                );
+                break;
+            }
+            case Type::Image:
+            {
+#ifndef QMVK_NO_GRAPHICS
+                auto image = static_pointer_cast<Image>(object);
+                image->pipelineBarrier(
+                    commandBuffer,
+                    descriptorInfos()[descriptorInfosIdx].descrImgInfo.imageLayout,
+                    pipelineStageFlags,
+                    accessFlag
+                );
+                descriptorInfosIdx += (m_plane == ~0u)
+                    ? image->numPlanes()
+                    : 1
+                ;
+#endif
+                break;
+            }
+        }
     }
 }
-void MemoryObjectDescr::finalizeImage(
+void MemoryObjectDescr::finalizeObject(
     vk::CommandBuffer commandBuffer) const
 {
     if (m_type != Type::Image || m_access != Access::Write)
         return;
 
+#ifndef QMVK_NO_GRAPHICS
     for (auto &&object : m_objects)
     {
         auto image = static_pointer_cast<Image>(object);
         image->maybeGenerateMipmaps(commandBuffer);
     }
+#endif
 }
 
-MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getBufferDescriptorTypeInfos() const
+MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getBufferDescriptorTypeInfos(const vector<BufferRange> &ranges) const
 {
+    if (m_access == Access::Write)
+        throw vk::LogicError("Bad buffer access");
+
     DescriptorTypeInfos descriptorTypeInfos;
 
     auto &descriptorType = descriptorTypeInfos.first;
@@ -155,10 +194,11 @@ MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getBufferDescriptorTyp
     auto &descriptorInfos = descriptorTypeInfos.second;
     descriptorInfos.reserve(m_objects.size());
 
+    size_t i = 0;
     for (auto &&object : m_objects)
     {
         auto buffer = static_pointer_cast<Buffer>(object);
-        auto type = (m_access == Access::Read || (m_access == Access::Undefined && (buffer->usage() & vk::BufferUsageFlagBits::eUniformBuffer)))
+        auto type = (m_access == Access::Read)
             ? vk::DescriptorType::eUniformBuffer
             : vk::DescriptorType::eStorageBuffer
         ;
@@ -168,15 +208,33 @@ MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getBufferDescriptorTyp
         else if (descriptorType.type != type)
             throw vk::LogicError("Inconsistent buffer types");
 
-        descriptorInfos.push_back({{*buffer, 0, buffer->size()}});
+        vk::DeviceSize offset = 0;
+        vk::DeviceSize size = buffer->size();
+        if (ranges.size() > i && ranges[i].second > 0)
+        {
+            if (ranges[i].first + ranges[i].second > size)
+                throw vk::LogicError("Buffer range exceeds the buffer size");
+
+            offset = ranges[i].first;
+            size = ranges[i].second;
+        }
+
+        descriptorInfos.push_back({{
+            *buffer,
+            offset,
+            size,
+        }});
+
+        ++i;
     }
 
     descriptorType.descriptorCount = descriptorInfos.size();
     return descriptorTypeInfos;
 }
+#ifndef QMVK_NO_GRAPHICS
 MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getImageDescriptorTypeInfos() const
 {
-    if ((m_access == Access::Undefined) == !m_sampler)
+    if (m_access == Access::Storage || m_access == Access::StorageRead || m_access == Access::StorageWrite || (m_sampler && m_access != Access::Read))
         throw vk::LogicError("Bad image access");
 
     DescriptorTypeInfos descriptorTypeInfos;
@@ -224,9 +282,10 @@ MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getImageDescriptorType
     descriptorType.descriptorCount = descriptorInfos.size();
     return descriptorTypeInfos;
 }
+#endif
 MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getBufferViewDescriptorTypeInfos() const
 {
-    if (m_access == Access::Undefined)
+    if (m_access == Access::Write)
         throw vk::LogicError("Bad buffer view access");
 
     DescriptorTypeInfos descriptorTypeInfos;
@@ -258,12 +317,34 @@ MemoryObjectDescr::DescriptorTypeInfos MemoryObjectDescr::getBufferViewDescripto
 
 bool MemoryObjectDescr::operator ==(const MemoryObjectDescr &other) const
 {
-    return (
-        m_type == other.m_type &&
-        m_objects == other.m_objects &&
-        m_access == other.m_access &&
-        m_sampler == other.m_sampler && m_plane == other.m_plane
-    );
+    bool ret =
+           m_type == other.m_type
+        && m_objects == other.m_objects
+        && m_access == other.m_access
+#ifndef QMVK_NO_GRAPHICS
+        && m_sampler == other.m_sampler && m_plane == other.m_plane
+#endif
+    ;
+    if (ret && m_type == Type::Buffer)
+    {
+        const auto size = m_descriptorTypeInfos.second.size();
+        for (size_t i = 0; i < size; ++i)
+        {
+            const auto &descriptorInfo = m_descriptorTypeInfos.second[i];
+            const auto &descriptorInfoOther = other.m_descriptorTypeInfos.second[i];
+            if (descriptorInfo.descrBuffInfo.offset != descriptorInfoOther.descrBuffInfo.offset)
+            {
+                ret = false;
+                break;
+            }
+            if (descriptorInfo.descrBuffInfo.range != descriptorInfoOther.descrBuffInfo.range)
+            {
+                ret = false;
+                break;
+            }
+        }
+    }
+    return ret;
 }
 
 }
