@@ -1,19 +1,7 @@
+// SPDX-License-Identifier: MIT
 /*
-    QmVk - simple Vulkan library created for QMPlay2
-    Copyright (C) 2020-2021  Błażej Szczygieł
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published
-    by the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   QmVk - simple Vulkan library created for QMPlay2
+   Copyright (C) 2020-2022 Błażej Szczygieł
 */
 
 #include "MemoryObjectDescrs.hpp"
@@ -56,18 +44,18 @@ vector<DescriptorInfo> MemoryObjectDescrs::fetchDescriptorInfos() const
     return descriptorInfos;
 }
 
-void MemoryObjectDescrs::prepareImages(
+void MemoryObjectDescrs::prepareObjects(
     vk::CommandBuffer commandBuffer,
     vk::PipelineStageFlags pipelineStageFlags) const
 {
     for (auto &&memoryObject : *m_memoryObjects)
-        memoryObject.prepareImage(commandBuffer, pipelineStageFlags);
+        memoryObject.prepareObject(commandBuffer, pipelineStageFlags);
 }
-void MemoryObjectDescrs::finalizeImages(
+void MemoryObjectDescrs::finalizeObjects(
     vk::CommandBuffer commandBuffer) const
 {
     for (auto &&memoryObject : *m_memoryObjects)
-        memoryObject.finalizeImage(commandBuffer);
+        memoryObject.finalizeObject(commandBuffer);
 }
 
 bool MemoryObjectDescrs::operator ==(const MemoryObjectDescrs &other) const
