@@ -1,19 +1,7 @@
+// SPDX-License-Identifier: MIT
 /*
-    QmVk - simple Vulkan library created for QMPlay2
-    Copyright (C) 2020-2021  Błażej Szczygieł
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published
-    by the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   QmVk - simple Vulkan library created for QMPlay2
+   Copyright (C) 2020-2022 Błażej Szczygieł
 */
 
 #include "SwapChain.hpp"
@@ -89,8 +77,19 @@ void SwapChain::init(CreateInfo &createInfo)
         }
     }
 
+#ifdef VK_USE_PLATFORM_WIN32_KHR
+    const bool exclusiveFullScreen =
+        createInfo.exclusiveFullScreen != vk::FullScreenExclusiveEXT::eDefault &&
+        m_device->enabledExtensions().count(VK_EXT_FULL_SCREEN_EXCLUSIVE_EXTENSION_NAME) > 0
+    ;
+#endif
+
     if (createInfo.imageCount == 0 && presentMode == vk::PresentModeKHR::eMailbox)
         createInfo.imageCount = 3;
+#ifdef VK_USE_PLATFORM_WIN32_KHR
+    else if (exclusiveFullScreen && createInfo.imageCount == 0 && surfaceCapabilities.minImageCount == 1)
+        createInfo.imageCount = 2;
+#endif
     createInfo.imageCount = max(createInfo.imageCount, surfaceCapabilities.minImageCount);
     if (surfaceCapabilities.maxImageCount > 0)
         createInfo.imageCount = min(createInfo.imageCount, surfaceCapabilities.maxImageCount);
@@ -128,10 +127,8 @@ void SwapChain::init(CreateInfo &createInfo)
     vkCreateInfo.oldSwapchain = *m_oldSwapChain;
 #ifdef VK_USE_PLATFORM_WIN32_KHR
     vk::SurfaceFullScreenExclusiveInfoEXT surfaceFullScreenExclusiveInfo;
-    if (
-        createInfo.exclusiveFullScreen != vk::FullScreenExclusiveEXT::eDefault &&
-        m_device->enabledExtensions().count(VK_EXT_FULL_SCREEN_EXCLUSIVE_EXTENSION_NAME) > 0
-    ) {
+    if (exclusiveFullScreen)
+    {
         surfaceFullScreenExclusiveInfo.fullScreenExclusive = createInfo.exclusiveFullScreen;
         vkCreateInfo.pNext = &surfaceFullScreenExclusiveInfo;
     }
